Attribute VB_Name = "Battlenet"
Option Explicit
DefInt I
DefStr S
DefLng L
Public Sub Connect(ctlSock As Control, strServer, Optional strPort = 6112)
    'Connect to a Battle.net Server
    
    'ctlSock  : A Microsoft Winsock Control v5.0+
    'strServer: The Battle.net server
    'strPort  : The remote port (don't change this unless you know what you're doing)
    
    If ctlSock.State <> 0 Then ctlSock.Disconnect
    With ctlSock
        .RemoteHost = strServer
        .RemotePort = strPort
        .Protocol = sckTCPProtocol
        .Connect
    End With
End Sub
Public Sub Login(ctlSock As Control, strUser, strPass)
    'Login to the Battle.net Server. This should be called once a
    'connection has been established. The event Winsock_Connect
    'indicates that a connection has been established.
    
    'ctlSock    : The Winsock Control which established the connection
    'strUser    : Your username (use GUEST or ANONYMOUS if you don't have one)
    'strPassword: Your password (GUEST and ANONYMOUS don't require passwords)

    If ctlSock.State <> 0 Then
        ctlSock.SendData Chr(3) & Chr(4) & strUser & vbCrLf & strPass & vbCrLf
    End If
End Sub
Public Sub Disconnect(ctlSock As Control)
    'Disconnect from Battle.net
    
    'ctlSock: The Winsock Control
    ctlSock.Close
End Sub
Public Sub ParseData(strData)
    'Parse a data string which has been received from Battle.net.
    'This should be called whenever a Winsock_DataArrival event
    'occurs, where strData is the received data string.
    
    Dim strStrings() As String, I
    
    'Sometimes Battle.net puts several strings into one line,
    'separated by CrLf's. Here we split and parse them.
    
    SplitStr strData, strStrings, vbCrLf
    For I = 1 To UBound(strStrings)
        Parse2 strStrings(I)
    Next I
End Sub
Private Sub Parse2(ByVal strData As String)
    'This subroutine does the actual parsing. It gets called
    'by the ParseData subroutine. There is no need to call it
    'from outside.
    
    Dim strData1, strData2, I
    Dim strArgs() As String
    
    'Sometimes, Battle.net puts a LineFeed character at the
    'beginning of strings. We need to trim it.
    If Left(strData, 1) = Chr(10) Then
        strData = Right(strData, Len(strData) - 1)
    End If
    
    'Check whether this is normal text, or an event
    If Val(Left(strData, 4)) < 1001 Or Val(Left(strData, 4)) > 3000 Then
        Event_Unknown strData
        Exit Sub
    End If
    
    'If this is an event, then we separate the arguments, and quoted text
    I = InStr(1, strData, Chr(34), vbBinaryCompare)
    If I <> 0 Then
        strData1 = Left(strData, I - 2)
        strData2 = Mid(strData, I + 1, Len(strData) - I - 1)
    Else
        strData1 = strData
        strData2 = ""
    End If
    SplitStr strData1, strArgs(), " "
    
    'Call Appropriate Event based on ID
    Select Case Val(strArgs(1))
        Case 1001: Event_User strArgs(3), strArgs(4), strArgs(5), strArgs(6)
        Case 1002: Event_Join strArgs(3), strArgs(4), strArgs(5), strArgs(6)
        Case 1003: Event_Leave strArgs(3)
        Case 1004: Event_RecvWhisper strArgs(3), strArgs(4), strData2
        Case 1005: Event_Talk strArgs(3), strArgs(4), strData2
        Case 1006: Event_Broadcast strData2
        Case 1007: Event_Channel strData2
        Case 1009: Event_Flags strArgs(3), strArgs(4)
        Case 1010: Event_SendWhisper strArgs(3), strArgs(4), strData2
        Case 1013: Event_ChannelFull strData2
        Case 1014: Event_ChannelNotExist strData2
        Case 1015: Event_ChannelRestricted strData2
        Case 1016: Event_Info strData2
        Case 1018: Event_Info strData2
        Case 1019: Event_Error strData2
        Case 1023: Event_Emote strArgs(3), strArgs(4), strData2
        Case 2010: Event_Name strArgs(3)
        Case 3000: Event_Info strData2
    End Select
End Sub
Public Function SplitStr(ByVal OriginalString As String, ByRef ReturnArray() As String, ByVal Delimiter As String) As Long
    'Used to split a string by delimiters, into a dynamic array, and returns
    'the ubound of the array
    
    Dim sItem, lArrCnt
    Dim lLen, lPos
    
    lArrCnt = 0
    lLen = Len(OriginalString)

    Do
        lPos = InStr(1, OriginalString, Delimiter, vbTextCompare)
        If lPos <> 0 Then
            sItem = Left$(OriginalString, lPos - 1)
            OriginalString = Mid$(OriginalString, lPos + 1)
            If sItem <> "" Then
                lArrCnt = lArrCnt + 1
                If lArrCnt = 1 Then
                    ReDim ReturnArray(1 To lArrCnt) As String
                Else
                    ReDim Preserve ReturnArray(1 To lArrCnt) As String
                End If
                ReturnArray(lArrCnt) = sItem
            End If
        End If
    Loop While lPos <> 0

    If OriginalString <> "" Then
        lArrCnt = lArrCnt + 1
        ReDim Preserve ReturnArray(1 To lArrCnt) As String
        ReturnArray(lArrCnt) = OriginalString
    End If
    
    SplitStr = lArrCnt
End Function
Private Sub Event_Unknown(strText)
    'Normal text from the server (not an event)
    'strText: Text Message
    'ShowTxt strText & vbCrLf
End Sub
Private Sub Event_User(strUser, strFlag, strProduct, strPing)
    'Listing of a user in the current channel (after a Channel event)
    'strUser   : User's Name
    'strFlag   : User's Status Flag
    'strProduct: User's Product
    'strPing   : User's Ping
    If InStr(strUser, "*") Then sOps.IsD2 = True
    ShowTxt strUser & " - " & strProduct & " - " & strPing & vbCrLf
    If strUser = strName And strFlag = "0002" Then
    sOps.IsOps = True
    End If
    Dim strA As cList, intA As Integer, Safe As Boolean, suL As uL
    If LUsr(strUser).Status <> "0" And LUsr(strUser).Status < vbNullString Then
    suL.Username = strUser
    suL.LastSeen = Date & "|" & OpsTime
    End If
    Safe = False
    intA = GetChannelIndex(strUser)
    If intA <> -1 Then
    Channel(intA).Joined = GetTickCount()
    Channel(intA).Product = strProduct
    Channel(intA).Ping = strPing
    Channel(intA).InChannel = True
    Channel(intA).Flags = strFlag
    Else
    strA.Username = strUser
    strA.Joined = GetTickCount()
    strA.InChannel = True
    strA.Product = strProduct
    strA.Ping = strPing
    strA.Flags = strFlag
    AddChannel strA
    End If
End Sub
Private Sub Event_Join(strUser, strFlag As String, strProduct As String, strPing As String)
    'User joins current channel
    'strUser   : User's Name
    'strFlag   : User's Status Flag
    'strProduct: User's Product
    'strPing   : User's Ping
    'ShowTxt strUser & " joined using " & strProduct & vbCrLf
    Dim strA As cList, intA As Integer, Safe As Boolean, suL As uL, X As String, I As Long, dUser As uL
    dUser = LUsr(strUser)
    Safe = False
    If dUser.Status <> "0" And dUser.Status <> vbNullString Then
    If InStr(LCase(dUser.Flags), "s") Then Safe = True
    X = dUser.Greet
    If sOps.Greet = True Then
        If X = "" Then
            If LUsr(strUser).Rank <> "" Then
                DoEvents
                QueueAdd " Welcome " & FixName(strUser) & " (" & dUser.Rank & ")"
            End If
        Else
        X = Replace(X, "%u", FixName(strUser))
        X = Replace(X, "%r", dUser.Rank)
        X = Replace(X, "%a", dUser.Access)
        X = Replace(X, "%c", dUser.AddedOn)
        X = Replace(X, "%b", dUser.AddedBy)
        X = Replace(X, "%f", dUser.Flags)
        DoEvents
        QueueAdd X
        End If
    End If
    suL.Username = strUser
    suL.LastSeen = Date & "|" & OpsTime
    SUsr suL
    End If
    intA = GetChannelIndex(strUser)
    If intA <> -1 Then
        If (GetTickCount() - CLng(Channel(intA).Joined1)) <= 500 Then
            If Safe = False And sOps.JoinFloodProtect = True Then
            QueueAdd "/ban " & FixName(strUser) & " JFP"
            Channel(intA).Joined1 = Channel(intA).Joined
            Channel(intA).Joined = GetTickCount()
            Channel(intA).Product = strProduct
            Channel(intA).Ping = strPing
            Channel(intA).Banned = True
            Channel(intA).Flags = strFlag
            End If
        Else
            Channel(intA).Joined1 = Channel(intA).Joined
            Channel(intA).Joined = GetTickCount()
            Channel(intA).Product = strProduct
            Channel(intA).Ping = strPing
            Channel(intA).Flags = strFlag
        End If
    Else
        strA.Username = strUser
        strA.Joined = GetTickCount()
        strA.InChannel = True
        strA.Product = strProduct
        strA.Ping = strPing
        strA.Flags = strFlag
        AddChannel strA
    End If
    If Safe = False Then
        If sOps.IsOps = True Then
                intA = GetChannelIndex(FixName(strUser))
                If Channel(intA).Banned = True Then
                    QueueAdd "/ban " & strUser & " Ban Evasion"
                    Exit Sub
                End If
                If TagCheck(strUser) = True Then
                    Exit Sub
                End If
                If dUser.Banned = "1" Then
                    If dUser.Status = "1" = True Then
                    QueueAdd "/ban " & strUser & " Autoban"
                    QueueAdd "/ignore " & strUser
                    Exit Sub
                    End If
                End If
                If sOps.Lockdown = True Then
                    QueueAdd "/ban " & strUser & " Lockdown"
                    Exit Sub
                End If
        End If
    End If

    'ipban
    If strFlag = "0032" Then
        If sOps.IsOps = True Then
            If sOps.IPBan = True Then
                dUser = LUsr(strUser)
                If dUser.Status = "1" Then
                    If InStr(LCase(dUser.Flags), "s") Then
                        QueueAdd "/unignore " & FixName(strUser)
                        QueueAdd "User " & FixName(strUser) & " is safelisted therefore is now un-ipbanned."
                        Exit Sub
                    Else
                        intA = GetChannelIndex(FixName(strUser))
                        Channel(intA).IPBan = True
                        QueueAdd "/ban " & FixName(strUser) & " IPBan"
                    Exit Sub
                    End If
                Else
                    QueueAdd "/ban " & FixName(strUser) & " IPBan"
                    intA = GetChannelIndex(FixName(strUser))
                    Channel(intA).IPBan = True
                End If
            End If
        End If
    End If
End Sub
Private Sub Event_Leave(strUser)
    'User leaves current channel
    'strUser: User's Name
    'strFlag: User's Status Flag
    'ShowTxt strUser & " left" & vbCrLf
    Dim strA As cList, intA As Integer, Safe As Boolean
    Safe = False
    intA = GetChannelIndex(strUser)
    If intA <> -1 Then
        Channel(intA).InChannel = False
    End If
End Sub
Private Sub Event_RecvWhisper(strUser, strFlag, strText)
    'Received Whisper
    'strUser: User's Name
    'strFlag: User's Status Flag
    'strText: Whisper Text
    ShowTxt "<From: " & strUser & "> " & strText & vbCrLf
End Sub
Private Sub Event_Talk(strUser, strFlag, strText)
    'User Talks
    'strUser: User's Name
    'strFlag: User's Status Flag
    'strText: Whisper Text
    'ShowTxt "<" & strUser & "> " & strText & vbCrLf
    Dim dUser As uL
    Dim intA As Integer, suL As uL, Safe As Boolean, vAdd As Boolean, I As Integer
    If vBan.Status = True Then
    Select Case LCase(strText)
        Case "yay"
        If vBan.Users(0) = "" Then
        vBan.Users(0) = strUser
        vBan.Yay = vBan.Yay + 1
        Else
        If GetVBanIndex(strUser) = -1 Then
        ReDim Preserve vBan.Users(UBound(vBan.Users) + 1)
        vBan.Users(UBound(vBan.Users)) = strUser
        vBan.Yay = vBan.Yay + 1
        End If
        End If
        Case "nay"
        If vBan.Users(0) = "" Then
        vBan.Users(0) = strUser
        vBan.Nay = vBan.Nay + 1
        Else
        If GetVBanIndex(strUser) = -1 Then
        ReDim Preserve vBan.Users(UBound(vBan.Users) + 1)
        vBan.Users(UBound(vBan.Users)) = strUser
        vBan.Nay = vBan.Nay + 1
        End If
        End If
    End Select
    End If
    dUser = LUsr(strUser)
    If InStr(LCase(dUser.Flags), "s") And dUser.Status = "1" Then Safe = True
    intA = GetChannelIndex(FixName(strUser))
    If dUser.Status = "1" Then
        suL.Username = FixName(strUser)
        suL.LastSeen = Date & "|" & OpsTime
        SUsr suL
        If Int(LUsr(strUser).Access) > 0 Then
        Select Case LCase(strText)
        Case "?trigger": QueueAdd "This bot's current trigger is: " & strTrigger: Exit Sub
        Case "?whoami", "?me", "?iam": QueueAdd FixName(strUser) & " Rank: " & LUsr(strUser).Rank & " Flags: " & LUsr(strUser).Flags & " Access: " & LUsr(strUser).Access & " AddedOn: " & LUsr(strUser).AddedOn & " AddedBy: " & LUsr(strUser).AddedBy: Exit Sub
        Case "?ver", "?version", "?about": QueueAdd frmBot.Caption: Exit Sub
        Case Else: If Left(LCase(strText), Len(strTrigger)) = LCase(strTrigger) Then Commands.Commands strUser, Right(strText, Len(strText) - Len(strTrigger))
        End Select
        End If
    End If
        If Safe = False Then
            If sOps.JoinFloodProtect = True Then
                If GetTickCount() - Channel(intA).Joined <= 500 Then
                    If sOps.IsOps = True Then QueueAdd "/ban " & FixName(strUser) & " JFP"
                End If
            End If
                If sOps.Spamban = True Then
                If Channel(intA).LastTalk1 = 0 Then
                Channel(intA).LastTalk1 = GetTickCount()
                Else
                Channel(intA).LastTalk2 = Channel(intA).LastTalk1
                Channel(intA).LastTalk1 = GetTickCount()
                End If
                If GetTickCount() - CLng(Channel(intA).LastTalk2) <= 500 Then
                    QueueAdd "/ban " & FixName(strUser) & " Spamban"
                End If
                End If
        End If
ending:
            Channel(intA).Flags = strFlag
            Channel(intA).LastMessage = strText
        
End Sub
Private Sub Event_Broadcast(strText)
    'Broadcast from server administrator
    'strText: Broadcast Message
    ShowTxt "Broadcast Message: " & strText & vbCrLf
End Sub
Private Sub Event_Channel(strText)
Dim I As Integer
    'Joined New Channel
    'strText: Channel Name
    CurrentChannel = strText
    sOps.IsOps = False
    ShowTxt "Joined Channel: " & strText & vbCrLf
    If LCase(strText) <> LCase(OpsChannel) And FirstConnect = True Then
    QueueAdd "/join " & OpsChannel
    FirstConnect = False
    Else
    Bancount = 0
    For I = 0 To UBound(Channel)
    If Channel(I).InChannel = True Then Channel(I).InChannel = False
    Next I
    End If
End Sub
Private Sub Event_Flags(strUser, strFlag)
    'User Changes Status Flags
    'strUser: User's Name
    'strFlag: User's Status Flag
    'ShowTxt "Username: " & strUser & " Flag Update: " & strFlag & vbNewLine
    'ShowTxt strUser & " " & strFlag & vbNewLine
    Dim dUser As uL, intA As Integer
    If strUser = strName And strFlag = "0002" Then
    sOps.IsOps = True
    End If
    If strFlag = "0032" Then
        If sOps.IsOps = True Then
            If sOps.IPBan = True Then
                dUser = LUsr(strUser)
                If dUser.Status = "1" Then
                    If InStr(LCase(dUser.Flags), "s") Then
                    QueueAdd "/unignore " & FixName(strUser)
                    QueueAdd "User " & FixName(strUser) & " is safelisted therefore is now un-ipbanned."
                    Exit Sub
                    Else
                    Exit Sub
                    QueueAdd "/ban " & FixName(strUser) & " IPBan"
                    intA = GetChannelIndex(FixName(strUser))
                    Channel(intA).IPBan = True
                    End If
                Else
                QueueAdd "/ban " & FixName(strUser) & " IPBan"
                End If
            End If
        End If
    End If
End Sub
Private Sub Event_SendWhisper(strUser, strFlag, strText)
    'You Send a Whisper
    'strUser: User's Name
    'strFlag: User's Status Flag
    'strText: Whisper Text
    ShowTxt "<To: " & strUser & "> " & strText & vbCrLf
End Sub
Private Sub Event_ChannelFull(strText)
    'Channel is Full
    'strText: Name of Channel
    ShowTxt "Chanenl " & strText & " is full" & vbCrLf
    'If LCase(strText) <> LCase(OpsChannel) Then QueueAdd "/join " & OpsChannel
End Sub
Private Sub Event_ChannelNotExist(strText)
    'Channel does not Exist
    'strText: Name of Channel
    ShowTxt "Channel " & strText & " does not exist" & vbCrLf
End Sub
Private Sub Event_ChannelRestricted(strText)
    'Channel is Restricted
    'strText: Name of Channel
    ShowTxt "Channel " & strText & " is restricted" & vbCrLf
End Sub
Private Sub Event_Info(strText)
    'Information from the Server
    'strText: Info Message
    Dim Splt() As String, A As Integer, strA As cList
    If InStr(LCase(strText), "has been squelched") Then
        Splt() = Split(strText, " ", 2)
        A = GetChannelIndex(Splt(0))
            If A <> -1 Then
            A = GetChannelIndex(Splt(0))
            Channel(A).IPBan = True
            Bancount = Bancount - 1
            Else
                Bancount = Bancount - 1
                strA.Username = Splt(0)
                AddChannel strA
                A = GetChannelIndex(Splt(0))
                Channel(A).IPBan = True
            End If
    End If
    If InStr(LCase(strText), "has been unsquelched") Then
        Splt() = Split(strText, " ", 2)
        A = GetChannelIndex(Splt(0))
            If A <> -1 Then
            A = GetChannelIndex(Splt(0))
            Channel(A).IPBan = False
            Bancount = Bancount - 1
            Else
                Bancount = Bancount - 1
                strA.Username = Splt(0)
                AddChannel strA
                A = GetChannelIndex(Splt(0))
                Channel(A).IPBan = False
            End If
    End If
    If InStr(LCase(strText), "was unbanned") Then
        Splt() = Split(strText, " ", 2)
        A = GetChannelIndex(Splt(0))
            If A <> -1 Then
            A = GetChannelIndex(Splt(0))
            Channel(A).Banned = False
            Bancount = Bancount - 1
            Else
                Bancount = Bancount - 1
                strA.Username = Splt(0)
                AddChannel strA
                A = GetChannelIndex(Splt(0))
                Channel(A).Banned = False
            End If
    End If
    If InStr(LCase(strText), "was banned") Then
        Splt() = Split(strText, " ", 2)
        A = GetChannelIndex(Splt(0))
            If A <> -1 Then
            Channel(A).Banned = True
            Bancount = Bancount + 1
            Else
                Bancount = Bancount + 1
                strA.Username = Splt(0)
                AddChannel strA
                A = GetChannelIndex(Splt(0))
                Channel(A).Banned = True
            End If
    End If
    'If InStr(LCase(strText), "clan member: ") Then
    'Splt() = Split(strText, " ", 5)
    'Debug.Print Splt(1) & " " & Splt(4)
    'If LCase(Splt(4)) = "chieftan" Then
    'ShowTxt Splt(2) & " " & Splt(4) & vbNewLine
    'End If
    'End If
    ShowTxt strText & vbCrLf
End Sub
Private Sub Event_Error(strText)
    'Error from the Server
    'strText: Error Message
    ShowTxt strText & vbCrLf
End Sub
Private Sub Event_Emote(strUser, strFlag, strText)
    'Emote Event
    'strUser: User's Name
    'strFlag: User's Status Flag
    'ShowTxt "<" & strUser & " " & strText & ">" & vbCrLf
End Sub
Private Sub Event_Name(strUser)
    'Username Assignment
    'strUser: Name which Battle.net assigns to you (once during login)
    ShowTxt "Assigned Name: " & strUser & vbCrLf
    strName = strUser
End Sub
